import * as React from 'react';
import {observer} from "mobx-react-lite";
import {ModelCheckbox, ModelForm, ModelInput, useModelContext} from "@rx-suite/ui";
import {Box, Typography} from '@material-ui/core';
import {t, Trans} from "@lingui/macro";
import {DeviceTypeHint} from "../../../components/dcs_config/hints/DeviceTypeHint";
import {MaxOpenConnectionsHint} from "../../../components/dcs_config/hints/MaxConnectionsOpenHint";
import {DcsConfigModel} from "../../../stores/DcsConfigStore";
import {ConnectionTimoutHint} from "../../../components/dcs_config/hints/ConnectionTimeoutHint";

export const DcsConfigTab: React.FunctionComponent = observer(() => {
    const model = useModelContext() as DcsConfigModel;

    return (
        <React.Fragment>
            <ModelForm model={model} suppressImportExport suppressWrapper suppressMakeCopy>
                <ModelInput name={'name'}/>
                <ModelInput name={'description'}
                            hint={
                                {
                                    title: t`description`,
                                    text: t`Used to identify the DCS connection in lists.`
                                }
                            }/>
                <Box mt={1}>
                    <Typography variant={'h6'}>
                        <Trans>Connection Settings</Trans>
                    </Typography>
                </Box>
                <ModelInput name={'ip'}/>
                <ModelInput name={'port'}/>
                <ModelInput name={'device_type'}
                            hint={
                                    {
                                        title: t`Device Type`,
                                        children: <DeviceTypeHint/>

                                    }
                                }
                />
                <Box mt={1}>
                    <Typography variant={'h6'}>
                        <Trans>Advanced Connection Settings</Trans>
                    </Typography>
                </Box>
                <ModelInput name={'retry_on_fail_connection'}
                            type={'number'}
                            inputProps={
                                {
                                    min: 0,
                                    max: 99
                                }
                            }
                            hint={
                                {
                                    title: t`Retry attempts on failed connections`,
                                    text: t`How many time to retry the connection to the external system on a failed attempt.`
                                }
                            }
                />
                <ModelInput name={'connection_timeout'}
                            type={'number'}
                            inputProps={
                                {
                                    min: 3,
                                    max: 99
                                }
                            }
                            hint={
                                {
                                    title: t`Connection Timeout`,
                                    children: <ConnectionTimoutHint/>

                                }
                            }
                />
                <ModelInput name={'max_open_connections'}
                            type={'number'}
                            inputProps={
                                {
                                    min: 0,
                                    max: 99
                                }
                            }
                            hint={
                                {
                                    title: t`Maximum Open Connection`,
                                    children: <MaxOpenConnectionsHint/>

                                }
                            }
                />
                <ModelInput name={'delay_on_socket_open'}
                            type={'number'}
                            inputProps={
                                {
                                    min: 0,
                                    max: 99
                                }
                            }
                            hint={
                                    {
                                        title: t`Delay on Socket Open`,
                                        text: t`Delay between opening the connection to the external system and sending the INI message. Leave blank for no delay.`
                                    }
                            }
                />
                <Box>
                    <ModelCheckbox name={'hold_connection_open'}
                                   hint={
                                            {
                                                title: t`Hold Connection Open`,
                                                text: t`If disabled, a new socket connection is made with every DCS call. Otherwise the socket is kept open between calls.`
                                            }
                                    }
                    />
                </Box>
                <Box mt={1}>
                    <Typography variant={'h6'}>
                        <Trans>Other Settings</Trans>
                    </Typography>
                </Box>
                <Box mb={2}>
                    <ModelCheckbox name={'keep_call_trace'}/>
                </Box>
            </ModelForm>
            {
                !model.isNew &&
                <Box ml={1.5} mt={-0.5}>
                    <Typography variant={'caption'}>{t`Changes to connection settings are not applied to already opened connection. Please reinitialize the connection to apply changes.`}</Typography>
                </Box>
            }
        </React.Fragment>
    )
});
