import * as React from 'react';
import {observer} from "mobx-react-lite";
import {ButtonField, Conditional, IfFalse, IfTrue, InputField, useModelContext} from "@rx-suite/ui";
import {Box, Divider, makeStyles, Typography} from '@material-ui/core';
import {t, Trans,} from "@lingui/macro";
import {DcsConfigModel} from "../../../stores/DcsConfigStore";
import {DcsTagsTable} from "../../../components/dcs_config/DcsTagsTable";
import {useSnackbar} from "notistack";
import {DcsErrorData} from "../../../components/dcs_config/DcsErrorData";

const useStyles = makeStyles((theme) => ({
    hint: {
        fontSize: '0.75rem',
        marginTop: -1 * theme.spacing(0.5)
    }
}));

export const DcsTestTab: React.FunctionComponent = observer(() => {
    const model = useModelContext() as DcsConfigModel;

    const {enqueueSnackbar} = useSnackbar();
    const classes = useStyles();

    const [data, setData] = React.useState<any|null>(null);
    const [jobNumber, setJobNumber] = React.useState<string>('');
    const [tags, setTags] = React.useState<string>('');
    const [isSubmitting, setIsSubmitting] = React.useState<boolean>(false);

    const handleJobCall = async () => {
        setIsSubmitting(true);
        setData(null);
        try{
            const parsedTags = tags.split(',')
                                .map((tag: string) => tag.trim().toUpperCase())
                                .filter(tag => tag !== '');

            const resp = await model.performJobCall(jobNumber, parsedTags);

            if(resp['error']){
                enqueueSnackbar(`${t`Test call failed.`}`, {variant: "error"})
            }else{
                enqueueSnackbar(`${t`Test call successful.`}`, {variant: "success"})
            }

            setData(resp);
        }catch (err){
            console.error(err);
            setData(err);
            enqueueSnackbar(`${t`Test call failed.`}`, {variant: "error"})
        }
        setIsSubmitting(false);
    }

    return (
        <React.Fragment>
            <InputField
                label={t`Job Number`}
                onChange={(value)=>{setJobNumber(value.target.value)}}
                value={jobNumber}
            />
            <InputField
                label={t`Tags`}
                onChange={(value)=>{setTags((value.target.value))}}
                value={tags}
            />
            <div className={classes.hint}><Trans>Note: to test several tags, separate them with a comma (ex: LIND, CRIB). </Trans></div>
            <Box mt={1}>
                <ButtonField text={t`Test Connection`}
                             buttonType={'submit'}
                             onClick={handleJobCall}
                             color={'secondary'}
                             loading={isSubmitting}
                             disabled={jobNumber.length === 0}
                />
            </Box>
            {data &&
                <Box mt={1}>
                    <Divider/>
                    <Typography variant={'h5'}><Trans>Response</Trans></Typography>
                    <Box mt={1}>
                        <Conditional condition={data['success']}>
                            <IfTrue>
                                <DcsTagsTable tagData={data['tags']}/>
                            </IfTrue>
                            <IfFalse>
                                <DcsErrorData error={data['error']}/>
                            </IfFalse>
                        </Conditional>
                    </Box>
                </Box>
            }
        </React.Fragment>
    )
});
