class DcsClientManager
  include Singleton

  class MissingJobNumber < StandardError; end

  def initialize
    @clients = {}
  end

  def execute_ping_for_connection(dcs_config)
    client = client_for_config(dcs_config)
    client.ping
  end

  def force_reset_for_connection(dcs_config)
    client = client_for_config(dcs_config)
    client.release_DCS_client
    client.load_config(dcs_config)
    client.ping
  end

  def execute_job_call_for_connection(dcs_config, job_call_data)
    raise MissingJobNumber if job_call_data[:job].nil?

    tags = []
    tags.concat job_call_data[:tags] unless job_call_data[:tags].nil?

    client = client_for_config(dcs_config)

    begin
      response = client.execute_job_call(job_call_data[:job], tags)
    rescue DcsClientWrapper::DcsClientTimeout
      return build_error_response('Timed out while fetching tags.', timeout: true)
    rescue DcsClientWrapper::DcsClientConnectionFail
      return build_error_response('Failed to connect to external system.', failToFetch: true)
    end

    return build_error_response('Received no data from external system.', raw_response: response, failToFetch: true) if response[:tags].nil?

    parsed_tags = split_tags_for_lenside(response[:tags])

    handle_response(response, parsed_tags)
  end

  private

  def client_for_config(dcs_config)
    @clients[dcs_config.ip] = {} if @clients[dcs_config.ip].nil?

    if @clients[dcs_config.ip][dcs_config.port].nil?
      @clients[dcs_config.ip][dcs_config.port] = DcsClientWrapper.new(dcs_config)
    end

    @clients[dcs_config.ip][dcs_config.port]
  end

  def split_tags_for_lenside(data)
    tags = {}

    do_left = do_left?(data)
    do_right = do_right?(data)

    tags[:left] = if do_left
                     {}
                   else
                     nil
                   end

    tags[:right] = if do_right
                      {}
                    else
                      nil
                    end

    data.each do |tag, value|
      if value.include? ';'
        right_tag_val, left_tag_val = value.split(';')
      else
        left_tag_val = value
        right_tag_val = value
      end

      if do_left
        tags[:left][tag] = left_tag_val
      end
      if do_right
        tags[:right][tag] = right_tag_val
      end
    end

    tags
  end

  def do_right?(tags)
    false if tags.nil?
    do_both?(tags) || tags['DO'] == 'R'
  end

  def do_left?(tags)
    false if tags.nil?
    do_both?(tags) || tags['DO'] == 'L'
  end

  def do_both?(tags)
    false if tags.nil?
    tags['DO'] == 'B'
  end

  def handle_response(raw_response, parsed_tags)
    return build_error_response('Response missing STATUS tag!', raw_response: raw_response, failToFetch: true) if raw_response[:tags]['STATUS'].nil?

    status_code, status_data = raw_response[:tags]['STATUS'].split(';')

    if status_code != '0'
      error = if status_data
       status_data
      else
       'Unknown Error!'
      end
      build_error_response(error, raw_response: raw_response, jobDoesNotExist: true)
    else
      {
        success: true,
        tags: parsed_tags,
        raw_response: raw_response
      }
    end
  end

  def build_error_response(error_message, raw_response: nil, timeout: false, failToFetch: false, jobDoesNotExist: false)
    {
      success: false,
      error: {
        message: error_message,
        timeout: timeout,
        failToFetch: failToFetch,
        jobDoesNotExist: jobDoesNotExist
      },
      raw_response: raw_response
    }
  end
end
