class DcsClientWrapper
  class DcsClientTimeout < StandardError; end
  class DcsClientConnectionFail < StandardError; end

  def initialize(dcs_config)
    @client = nil
    @use_count = 0
    @mutex = Mutex.new

    load_config(dcs_config)
  end

  def execute_job_call(job_number, tags)
    attempts = 0
    resp = {}

    log('Attempting to acquire mutex...')
    checkout_client

    @mutex.synchronize do
      begin
        ensure_client
        resp = {tags: get_data_from_client(job_number, tags)}
        release_DCS_client unless @configs[:hold_connection_open]
      rescue Errno::ETIMEDOUT # Timeout from connection timeout
        attempts += 1
        log("Timed out while fetching tags for job #{job_number}. Attempt #{attempts}/#{@configs[:max_retry]+1}.")
        retry if can_retry?(attempts)
        raise DcsClientTimeout
      rescue IOError, Errno::ECONNRESET, Errno::EPIPE # Connection is closed from the lms side. Need to reestablish connection
        attempts += 1
        log("Lost connection while fetching tags for job #{job_number}. Attempt #{attempts}/#{@configs[:max_retry]+1}.")
        release_DCS_client
        retry if can_retry?(attempts)
        raise DcsClientConnectionFail
      ensure
        checkin_client
      end
    end

    log('Mutex Released')
    
    resp
  end

  def ping
    checkout_client

    success = true

    begin
      @mutex.synchronize do
        ensure_client
        release_DCS_client unless @configs[:hold_connection_open]
      end
    rescue
      success = false
    end

    checkin_client

    success
  end

  def release_DCS_client
    return if @client.nil?
    force_close_client
    @client = nil
  end

  def load_config(dcs_config)
    @ip = dcs_config.ip
    @port = dcs_config.port
    @device_type = dcs_config.device_type

    @configs = {
      connection_timeout: dcs_config.connection_timeout,
      max_retry: dcs_config.retry_on_fail_connection,
      max_open_connections: dcs_config.max_open_connections,
      socket_open_delay: dcs_config.delay_on_socket_open,
      hold_connection_open: dcs_config.hold_connection_open
    }
  end

  private

  def get_data_from_client(job_number, tags)
    @client.set_tags(tags)
    log("Executing Job Call for #{job_number}")

    tag_values = {}
    @client.get_job_info(job_number) do |data|
      tag_name = data[0]
      tag_values[tag_name] = data[1]
    end

    log("Received tags for job #{job_number}: #{tag_values}")

    tag_values
  end

  def checkout_client
    log("Connections: #{@use_count} => #{@use_count+1}")

    @use_count += 1

    if close_on_too_many_open_connections? && @use_count > @configs[:max_open_connections]
      log("WARNING MAXIMUM CONNECTION THRESHOLD EXCEEDED, ATTEMPTING TO RELEASE CONNECTIONS.")
      force_close_client
    end
  end

  def checkin_client
    log("Connections: #{@use_count} => #{@use_count-1}")

    @use_count -= 1
  end

  def ensure_client
    if @client.nil?
      log("Initializing client...")
      @client = DcsClient::DcsClient.new(@ip, @port, @configs[:connection_timeout])
      @client.dev_type = @device_type
      socket_open_delay
    else
      log("Client already initialized")
    end
  end

  def socket_open_delay
    return unless use_socket_open_delay?
    sleep @configs[:socket_open_delay]
  end

  def force_close_client
    log("Closing Client")
    begin
      @client.close
    rescue
      # close can throw for a number of reason, let it be since the connection will be dead regardless.
      # forcing the client to close will also consequently error out any active call that could be holding up the mutex
    end
  end

  def close_on_too_many_open_connections?
    return false if @configs[:max_open_connections].nil?
    true
  end

  def can_retry?(current_attempts)
    current_attempts <= @configs[:max_retry]
  end

  def use_socket_open_delay?
    return false if @configs[:socket_open_delay].nil? || @configs[:socket_open_delay] < 1
    true
  end

  def hostname
    "#{@ip}:#{@port}"
  end

  def log(message)
    Rails.logger.info "[DCS Client #{hostname} - dev_type: #{@device_type}] #{message}"
  end
end
