require "socket"
require "singleton"

class Dcs2restServer
  include Singleton

  class MissingMachine < StandardError; end
  class MachineConfigNotImplemented < StandardError; end

  def initialize
    @server = nil
    @listener_thread = nil
    @clients = Set.new
  end

  def start(port)
    unless @server.nil?
      Rails.logger.warn "Dcs2Rest Server is already started."
      return
    end

    Dcs2restServer.log "Starting server on port #{port}"

    @server = TCPServer.new("0.0.0.0", port)
    @listener_thread = Thread.new do
      Rails.application.executor.wrap do
        loop do
          # Stop listening for new connections if the server is stopped
          break if @server.nil?
          @clients << Thread.new(@server.accept) do |server_client|
            Rails.application.executor.wrap do
              ip_address = request_ip(server_client)
              request_time = Time.zone.now

              Dcs2restServer.log "incoming connection from #{ip_address}"

              begin
                data = read_socket(server_client)
              rescue => err
                return Dcs2restServer.log("Could not read from the Socket - #{err}", :error)
              end

              begin
                machine_config = ensure_machine_config(ip_address)
                machine = ensure_machine(machine_config.rx_data_machine_id)
              rescue => err
                return Dcs2restServer.log("Error while translating message - #{err}", :error)
              end

              begin
                interface = InterfaceConfig.select_interface(machine_config, data)

                xml = interface.dcs_to_xml(data)

                send_data(interface.endpoint, machine, request_time, xml)

                machine_config.create_trace(data, request_time, xml, interface.endpoint)
              rescue => err
                Dcs2restServer.log("Error while translating message - #{err}", :error)
                machine_config.create_error_trace(data, request_time, err)
              end
            ensure
              @clients.delete Thread.current
            end
          end
        end
      end
    end
  end

  def stop
    return if @server.nil?

    Dcs2restServer.log "Stopping server"
    Dcs2restServer.log "Waiting for #{@clients.count} processes to finish..."

    wait_for_clients_to_finish

    Dcs2restServer.log "Finished processing connections"

    kill_accept_loop
    close_server

    Dcs2restServer.log "Server stopped"
  end

  def online?
    @server.present?
  end

  private

  def read_socket(client)
    data = []
    while (message = client.gets)
      message = message.chop
      Dcs2restServer.log("[Dcs2Rest Server] message: #{message}", :debug)
      data << message
    end

    begin
      client.close
    rescue => err
      Dcs2restServer.log "Failed to close socket to client - #{err}"
    end

    data
  end

  def self.log(msg, level=:info)
    Rails.logger.send(level, "[Dcs2Rest Server] #{msg}")
  end

  def wait_for_clients_to_finish
    @clients.each(&:join)
  end

  def kill_accept_loop
    @listener_thread&.kill
    @listener_thread = nil
  end

  def close_server
    @server&.close
    @server = nil
  end

  def request_ip(client)
    client.peeraddr.nil? ? "unknown" : client.peeraddr[3]
  end
  def ensure_machine_config(ip_address)
    machine_config = MachineConfig.where(ip_address: ip_address).first
    raise MachineConfigNotImplemented if machine_config.nil?
    machine_config
  end

  def ensure_machine(machine_id)
    machine = DataFetcher.machines.find {|m| m[:id] == machine_id}
    raise MissingMachine if machine.nil?
    machine
  end

  def send_data(endpoint, machine, time, xml)
    parameters = {
      sn: machine[:sn],
      date: time.to_date.to_s,
      time: time.strftime("%H:%M:%S")
    }

    p = parameters.map do |k,v|
      "#{k}=#{v}"
    end.join('&')
    e = endpoint + "?" + p

    if xml.is_a? Array
      xml.each do |x|
        SendToData.perform_later(e, x.to_xml)
      end
    else
      SendToData.perform_later(e, xml.to_xml)
    end
  end
end