class Dcs2restConfig < RxModule::ApplicationRecord
  has_many :interface_configs
  has_many :machine_configs

  class << self
    def default_config_json
      'public/DefaultInterfaceConfigs.json'
    end

    def create_default_config
      if Dcs2restConfig.where(name: default_name).count > 0
        return puts "DCS2REST defaults configurations already created - skipping"
      end

      puts "Seeding DCS2REST default configurations"

      require "json"
      file = File.open default_config_json
      data = JSON.load file

      config = Dcs2restConfig.new({
                           name: default_name,
                           machine_state_min: 200,
                           machine_state_max: 299,
                           job_state_min: 300,
                           job_state_max: 399,
                           error_min: 400,
                           error_max: 499,
                           maintenance_min: 500,
                           maintenance_max: 599,
                           calibration_min: 600,
                           calibration_max: 699,
                           event_min: 700,
                           event_max: 799
                         })

      config.add_new_interfaces_from_data(data)
      config.save!
    end

    def default_name
      'Default Configuration'
    end
  end

  def add_new_interfaces_from_data(data)
    data.each do |interface_type, interface_config_arr|
      interface_class = self.interface_type_to_class(interface_type.to_sym)
      interface_config_arr.each do |interface_config|
        interface_config.symbolize_keys!
        interface_config[:data] = interface_config[:data].to_json
        # add_to_target adds to the collection without saving.
        association(:interface_configs).add_to_target(interface_class.new(interface_config))
      end
    end
    # add a single process parameter interface
    association(:interface_configs).add_to_target(ProcessParameterInterfaceConfig.new(data: '{}'))
  end

  def interface_configs_as_hash
    interface_configs_hash = {
      machine_state: [],
      error: [],
      job_state: [],
      process_parameter: [],
      maintenance: [],
      calibration: [],
      event: []
    }
    interface_configs.each do |interface_config|
      interface_type = interface_class_to_type(interface_config.type.to_sym)
      interface_configs_hash[interface_type].push({
                                                    id: interface_config[:id],
                                                    x_status: interface_config[:x_status],
                                                    data: JSON.parse(interface_config[:data])
                                                  })
    end
    interface_configs_hash
  end

  def interface_type_to_class(interface_type)
    {
      machine_state: MachineStateInterfaceConfig,
      error: ErrorInterfaceConfig,
      job_state: JobStateInterfaceConfig,
      process_parameter: ProcessParameterInterfaceConfig,
      maintenance: MaintenanceInterfaceConfig,
      calibration: CalibrationInterfaceConfig,
      event: EventInterfaceConfig
    }[interface_type]
  end

  def interface_class_to_type(interface_class)
    {
      MachineStateInterfaceConfig: :machine_state,
      ErrorInterfaceConfig: :error,
      JobStateInterfaceConfig: :job_state,
      ProcessParameterInterfaceConfig: :process_parameter,
      MaintenanceInterfaceConfig: :maintenance,
      CalibrationInterfaceConfig: :calibration,
      EventInterfaceConfig: :event
    }[interface_class]
  end

  def connected_machine_ids
    machine_configs.pluck(:rx_data_machine_id)
  end

  def to_json(*args)
    render_jbuilder_partial
  end
end
